@@ -327,7 +327,7 @@ typedef enum {
     S2N_ERR_ATOMIC,
     S2N_ERR_KTLS_KEY_LIMIT,
     S2N_ERR_SECURITY_POLICY_INCOMPATIBLE_CERT,
-    S2N_INVALID_SERIALIZED_CONNECTION,
+    S2N_ERR_INVALID_SERIALIZED_CONNECTION,
     S2N_ERR_T_USAGE_END,
 } s2n_error;
 
