@@ -110,8 +110,7 @@ const struct s2n_signature_scheme s2n_ecdsa_sha256 = {
     .hash_alg = S2N_HASH_SHA256,
     .sig_alg = S2N_SIGNATURE_ECDSA,
     .libcrypto_nid = NID_ecdsa_with_SHA256,
-    .signature_curve = NULL,               /* Decided by supported_groups Extension in TLS 1.2 and before */
-    .maximum_protocol_version = S2N_TLS12, /* TLS1.3 requires a signature curve */
+    .signature_curve = &s2n_ecc_curve_secp256r1, /* Hardcoded for TLS 1.3 */
 };
 
 const struct s2n_signature_scheme s2n_ecdsa_sha384 = {
@@ -119,8 +118,7 @@ const struct s2n_signature_scheme s2n_ecdsa_sha384 = {
     .hash_alg = S2N_HASH_SHA384,
     .sig_alg = S2N_SIGNATURE_ECDSA,
     .libcrypto_nid = NID_ecdsa_with_SHA384,
-    .signature_curve = NULL,               /* Decided by supported_groups Extension in TLS 1.2 and before */
-    .maximum_protocol_version = S2N_TLS12, /* TLS1.3 requires a signature curve */
+    .signature_curve = &s2n_ecc_curve_secp384r1, /* Hardcoded for TLS 1.3 */
 };
 
 const struct s2n_signature_scheme s2n_ecdsa_sha512 = {
@@ -128,38 +126,7 @@ const struct s2n_signature_scheme s2n_ecdsa_sha512 = {
     .hash_alg = S2N_HASH_SHA512,
     .sig_alg = S2N_SIGNATURE_ECDSA,
     .libcrypto_nid = NID_ecdsa_with_SHA512,
-    .signature_curve = NULL,               /* Decided by supported_groups Extension in TLS 1.2 and before */
-    .maximum_protocol_version = S2N_TLS12, /* TLS1.3 requires a signature curve */
-};
-
-/* TLS 1.3 Compatible ECDSA Schemes */
-/* In TLS 1.3 the two byte IANA value also defines the Curve to use for signing */
-
-const struct s2n_signature_scheme s2n_ecdsa_secp256r1_sha256 = {
-    .iana_value = TLS_SIGNATURE_SCHEME_ECDSA_SECP256R1_SHA256,
-    .hash_alg = S2N_HASH_SHA256,
-    .sig_alg = S2N_SIGNATURE_ECDSA,
-    .libcrypto_nid = NID_ecdsa_with_SHA256,
-    .signature_curve = &s2n_ecc_curve_secp256r1, /* Hardcoded as of TLS 1.3 */
-    .minimum_protocol_version = S2N_TLS13,
-};
-
-const struct s2n_signature_scheme s2n_ecdsa_secp384r1_sha384 = {
-    .iana_value = TLS_SIGNATURE_SCHEME_ECDSA_SECP384R1_SHA384,
-    .hash_alg = S2N_HASH_SHA384,
-    .sig_alg = S2N_SIGNATURE_ECDSA,
-    .libcrypto_nid = NID_ecdsa_with_SHA384,
-    .signature_curve = &s2n_ecc_curve_secp384r1, /* Hardcoded as of TLS 1.3 */
-    .minimum_protocol_version = S2N_TLS13,
-};
-
-const struct s2n_signature_scheme s2n_ecdsa_secp521r1_sha512 = {
-    .iana_value = TLS_SIGNATURE_SCHEME_ECDSA_SECP521R1_SHA512,
-    .hash_alg = S2N_HASH_SHA512,
-    .sig_alg = S2N_SIGNATURE_ECDSA,
-    .libcrypto_nid = NID_ecdsa_with_SHA512,
-    .signature_curve = &s2n_ecc_curve_secp521r1, /* Hardcoded as of TLS 1.3 */
-    .minimum_protocol_version = S2N_TLS13,
+    .signature_curve = &s2n_ecc_curve_secp521r1, /* Hardcoded for TLS 1.3 */
 };
 
 /**
@@ -228,11 +195,9 @@ const struct s2n_signature_scheme* const s2n_sig_scheme_pref_list_20140601[] = {
     &s2n_rsa_pkcs1_sha512,
     &s2n_rsa_pkcs1_sha224,
 
-    /* ECDSA - TLS 1.2 */
-    &s2n_ecdsa_sha256, /* same iana value as TLS 1.3 s2n_ecdsa_secp256r1_sha256 */
-    &s2n_ecdsa_secp256r1_sha256,
-    &s2n_ecdsa_sha384, /* same iana value as TLS 1.3 s2n_ecdsa_secp384r1_sha384 */
-    &s2n_ecdsa_secp384r1_sha384,
+    /* ECDSA */
+    &s2n_ecdsa_sha256,
+    &s2n_ecdsa_sha384,
     &s2n_ecdsa_sha512,
     &s2n_ecdsa_sha224,
 
@@ -257,11 +222,9 @@ const struct s2n_signature_scheme* const s2n_sig_scheme_pref_list_20200207[] = {
     &s2n_rsa_pkcs1_sha512,
     &s2n_rsa_pkcs1_sha224,
 
-    /* ECDSA - TLS 1.2 */
-    &s2n_ecdsa_sha256, /* same iana value as TLS 1.3 s2n_ecdsa_secp256r1_sha256 */
-    &s2n_ecdsa_secp256r1_sha256,
-    &s2n_ecdsa_sha384, /* same iana value as TLS 1.3 s2n_ecdsa_secp384r1_sha384 */
-    &s2n_ecdsa_secp384r1_sha384,
+    /* ECDSA */
+    &s2n_ecdsa_sha256,
+    &s2n_ecdsa_sha384,
     &s2n_ecdsa_sha512,
     &s2n_ecdsa_sha224,
 
@@ -280,9 +243,9 @@ const struct s2n_signature_scheme* const s2n_sig_scheme_pref_list_default_fips[]
     &s2n_rsa_pkcs1_sha384,
     &s2n_rsa_pkcs1_sha512,
 
-    /* ECDSA - TLS 1.2 */
-    &s2n_ecdsa_sha256, /* same iana value as TLS 1.3 s2n_ecdsa_secp256r1_sha256 */
-    &s2n_ecdsa_sha384, /* same iana value as TLS 1.3 s2n_ecdsa_secp384r1_sha384 */
+    /* ECDSA */
+    &s2n_ecdsa_sha256,
+    &s2n_ecdsa_sha384,
     &s2n_ecdsa_sha512,
     &s2n_ecdsa_sha224,
 };
@@ -307,15 +270,10 @@ const struct s2n_signature_scheme* const s2n_sig_scheme_pref_list_20230317[] = {
     &s2n_rsa_pkcs1_sha512,
 
     /* TLS1.2 with ECDSA */
-    &s2n_ecdsa_sha256, /* same iana value as TLS 1.3 s2n_ecdsa_secp256r1_sha256 */
-    &s2n_ecdsa_sha384, /* same iana value as TLS 1.3 s2n_ecdsa_secp384r1_sha384 */
+    &s2n_ecdsa_sha256,
+    &s2n_ecdsa_sha384,
     &s2n_ecdsa_sha512,
 
-    /* TLS1.3 with ECDSA */
-    &s2n_ecdsa_secp256r1_sha256,
-    &s2n_ecdsa_secp384r1_sha384,
-    &s2n_ecdsa_secp521r1_sha512,
-
     /* TLS1.3 with RSA-PSS */
     &s2n_rsa_pss_pss_sha256,
     &s2n_rsa_pss_pss_sha384,
@@ -327,7 +285,6 @@ const struct s2n_signature_preferences s2n_signature_preferences_20230317 = {
     .signature_schemes = s2n_sig_scheme_pref_list_20230317,
 };
 
-/* Add s2n_ecdsa_secp521r1_sha512 */
 const struct s2n_signature_scheme* const s2n_sig_scheme_pref_list_20201021[] = {
     /* RSA PSS */
     &s2n_rsa_pss_pss_sha256,
@@ -343,13 +300,10 @@ const struct s2n_signature_scheme* const s2n_sig_scheme_pref_list_20201021[] = {
     &s2n_rsa_pkcs1_sha512,
     &s2n_rsa_pkcs1_sha224,
 
-    /* ECDSA - TLS 1.2 */
-    &s2n_ecdsa_sha256, /* same iana value as TLS 1.3 s2n_ecdsa_secp256r1_sha256 */
-    &s2n_ecdsa_secp256r1_sha256,
-    &s2n_ecdsa_sha384, /* same iana value as TLS 1.3 s2n_ecdsa_secp384r1_sha384 */
-    &s2n_ecdsa_secp384r1_sha384,
-    &s2n_ecdsa_sha512, /* same iana value as TLS 1.3 s2n_ecdsa_secp521r1_sha512 */
-    &s2n_ecdsa_secp521r1_sha512,
+    /* ECDSA */
+    &s2n_ecdsa_sha256,
+    &s2n_ecdsa_sha384,
+    &s2n_ecdsa_sha512,
     &s2n_ecdsa_sha224,
 
     /* SHA-1 Legacy */
@@ -393,11 +347,9 @@ const struct s2n_signature_scheme* const s2n_sig_scheme_pref_list_20201110[] = {
     &s2n_rsa_pkcs1_sha512,
     &s2n_rsa_pkcs1_sha224,
 
-    /* ECDSA - TLS 1.2 */
-    &s2n_ecdsa_sha256, /* same iana value as TLS 1.3 s2n_ecdsa_secp256r1_sha256 */
-    &s2n_ecdsa_secp256r1_sha256,
-    &s2n_ecdsa_sha384, /* same iana value as TLS 1.3 s2n_ecdsa_secp384r1_sha384 */
-    &s2n_ecdsa_secp384r1_sha384,
+    /* ECDSA */
+    &s2n_ecdsa_sha256,
+    &s2n_ecdsa_sha384,
     &s2n_ecdsa_sha512,
     &s2n_ecdsa_sha224,
 };
@@ -413,8 +365,8 @@ const struct s2n_signature_scheme* const s2n_sig_scheme_pref_list_20210816[] = {
     &s2n_rsa_pkcs1_sha384,
     &s2n_rsa_pkcs1_sha512,
 
-    /* ECDSA - TLS 1.2 */
-    &s2n_ecdsa_sha384, /* same iana value as TLS 1.3 s2n_ecdsa_secp384r1_sha384 */
+    /* ECDSA */
+    &s2n_ecdsa_sha384,
     &s2n_ecdsa_sha512,
 };
 
@@ -424,15 +376,12 @@ const struct s2n_signature_preferences s2n_signature_preferences_20210816 = {
 };
 
 const struct s2n_signature_scheme* const s2n_sig_scheme_pref_list_rfc9151[] = {
-    /* ECDSA - TLS 1.3 */
-    &s2n_ecdsa_secp384r1_sha384,
+    /* ECDSA */
+    &s2n_ecdsa_sha384,
 
     /* RSA PSS - TLS 1.3 */
     &s2n_rsa_pss_pss_sha384,
 
-    /* ECDSA - TLS 1.2 */
-    &s2n_ecdsa_sha384, /* same iana value as TLS 1.3 s2n_ecdsa_secp384r1_sha384 */
-
     /* RSA */
     &s2n_rsa_pss_rsae_sha384,
 
@@ -440,8 +389,8 @@ const struct s2n_signature_scheme* const s2n_sig_scheme_pref_list_rfc9151[] = {
 };
 
 const struct s2n_signature_scheme* const s2n_cert_sig_scheme_pref_list_rfc9151[] = {
-    /* ECDSA - TLS 1.3 */
-    &s2n_ecdsa_secp384r1_sha384,
+    /* ECDSA */
+    &s2n_ecdsa_sha384,
 
     /* RSA PSS
      * https://github.com/aws/s2n-tls/issues/3435
@@ -454,9 +403,6 @@ const struct s2n_signature_scheme* const s2n_cert_sig_scheme_pref_list_rfc9151[]
      * support rsa_pss.
      */
 
-    /* ECDSA - TLS 1.2 */
-    &s2n_ecdsa_sha384, /* same iana value as TLS 1.3 s2n_ecdsa_secp384r1_sha384 */
-
     /* RSA */
     &s2n_rsa_pkcs1_sha384,
 };
@@ -487,12 +433,9 @@ const struct s2n_signature_scheme* const s2n_sig_scheme_pref_list_test_all_fips[
     &s2n_rsa_pkcs1_sha224,
 
     /* ECDSA */
-    &s2n_ecdsa_sha256, /* same iana value as TLS 1.3 s2n_ecdsa_secp256r1_sha256 */
-    &s2n_ecdsa_secp256r1_sha256,
-    &s2n_ecdsa_sha384, /* same iana value as TLS 1.3 s2n_ecdsa_secp384r1_sha384 */
-    &s2n_ecdsa_secp384r1_sha384,
-    &s2n_ecdsa_sha512, /* same iana value as TLS 1.3 s2n_ecdsa_secp521r1_sha512 */
-    &s2n_ecdsa_secp521r1_sha512,
+    &s2n_ecdsa_sha256,
+    &s2n_ecdsa_sha384,
+    &s2n_ecdsa_sha512,
     &s2n_ecdsa_sha224,
 };
 
